@@ -1,5 +1,7 @@
 package Test::CheckManifest;
 
+# ABSTRACT: Check if your Manifest matches your distro
+
 use strict;
 use warnings;
 
@@ -10,7 +12,8 @@ use File::Basename;
 use Test::Builder;
 use File::Find;
 
-our $VERSION = '1.23';
+our $VERSION = '1.27';
+our $VERBOSE = 1;
 
 my $test      = Test::Builder->new();
 my $test_bool = 1;
@@ -34,18 +37,46 @@ sub import {
 }
 
 sub ok_manifest{
-    my ($hashref,$msg)    = @_;
+    my ($hashref,$msg) = @_;
     
     $test->plan(tests => 1) unless $plan;
     
     my $is_hashref = 1;
     $is_hashref = 0 unless ref($hashref);
     
-    $msg = $hashref unless $is_hashref;
+    unless ( $is_hashref ) {
+        $msg = $hashref;
+        $hashref = {};
+    }
+
+    my $tmp_path = dirname( File::Spec->rel2abs( $0 ) );
+
+    if ( $hashref->{file} ) {
+        $tmp_path = dirname $hashref->{file};
+    }
+    elsif ( $hashref->{dir} ) {
+        $tmp_path = $hashref->{dir};
+    }
     
-    my $bool     = 1;
-    my $home     = Cwd::realpath( dirname(File::Spec->rel2abs($0)) . '/..' );
-    my $manifest = Cwd::realpath( $home . '/MANIFEST' );
+    my $bool = 1;
+    my $home = Cwd::realpath( dirname $tmp_path );
+    my $manifest;
+
+    my $counter = 0;
+    while ( 1 ) {
+        my $manifest_path = File::Spec->catfile( $home . '/MANIFEST' );
+        last if -f $manifest_path;
+
+        my $tmp_home = Cwd::realpath( File::Spec->catdir( $home, '..' ) );
+
+        last if !$tmp_home || $tmp_home eq $home || $counter++ == 20;
+        $home = $tmp_home;
+    }
+
+    eval { $manifest = Cwd::realpath( $home . '/MANIFEST' ); 1; };
+    if ( !$manifest ) {
+        $test->BAILOUT( 'Cannot find a MANIFEST. Please check!' );
+    }
     
     my $skip;
     eval { $skip     = Cwd::realpath( $home . '/MANIFEST.SKIP' ); 1; };
@@ -84,13 +115,7 @@ sub ok_manifest{
         my $files_in_skip = _read_skip( $skip, \$msg, \$bool );
         last unless $files_in_skip;
             
-        my @files;
-        while( my $fh_line = <$fh> ){
-            if( $fh_line =~ /^\s*([^\s#]\S*)/ and $fh_line !~ /^META\.yml/ ){
-                push @files, $1;
-            }
-        }
-        #my @files = grep{!/^\s*$/ and !/^META\.yml/}<$fh>;
+        my @files = _read_file( $fh );
         close $fh;
     
         chomp @files;
@@ -153,9 +178,36 @@ sub ok_manifest{
     my $plus = 'The following files are not part of distro but named in the MANIFEST file: '.
                join(', ',@files_plus);
     
-    $test->is_num($test_bool,$bool,$msg);
-    $test->diag($diag) if scalar @missing_files >= 1 and $test_bool == 1;
-    $test->diag($plus) if scalar @files_plus    >= 1 and $test_bool == 1;
+    $test->is_num($bool,$test_bool,$msg);
+    $test->diag($diag) if scalar @missing_files >= 1 and $test_bool == 1 and $VERBOSE;
+    $test->diag($plus) if scalar @files_plus    >= 1 and $test_bool == 1 and $VERBOSE;
+}
+
+sub _read_file {
+    my ($fh) = @_;
+    
+    my @files;
+    while( my $fh_line = <$fh> ){
+        chomp $fh_line;
+        
+        next if $fh_line =~ m{ \A \s* \# }x;
+        next if $fh_line =~ m{# selftest};
+        
+        my ($file);
+        
+        if ( ($file) = $fh_line =~ /^'(\\[\\']|.+)+'\s*(.*)/) {
+            $file =~ s/\\([\\'])/$1/g;
+        }
+        else {
+            ($file) = $fh_line =~ /^(\S+)\s*(.*)/;
+        }
+
+        next unless $file;
+        
+        push @files, $file;
+    }
+    
+    return @files;
 }
 
 sub _not_ok_manifest{
@@ -166,8 +218,10 @@ sub _not_ok_manifest{
 
 sub _is_excluded{
     my ($file,$dirref,$filter,$bool,$files_in_skip,$home) = @_;
-    my @excluded_files = qw(pm_to_blib Makefile META.yml Build pod2htmd.tmp
-                            pod2htmi.tmp Build.bat .cvsignore MYMETA.json);
+    my @excluded_files = qw(
+        pm_to_blib Makefile META.yml Build pod2htmd.tmp LICENSE Makefile.PL
+        pod2htmi.tmp Build.bat .cvsignore MYMETA.json MYMETA.yml License Readme
+    );
 
     if ( $files_in_skip and 'ARRAY' eq ref $files_in_skip ) {
         (my $local_file = $file) =~ s{\Q$home\E/?}{};
@@ -205,24 +259,7 @@ sub _read_skip {
         return;
     }
     else {
-        while ( my $line = <$skip_fh> ) {
-            chomp $line;
-            
-            next if $line =~ m{ \A \s* \# }x;
-            
-            my ($file,$comment);
-            
-            if (($file, $comment) = $line =~ /^'(\\[\\']|.+)+'\s*(.*)/) {
-                $file =~ s/\\([\\'])/$1/g;
-            }
-            else {
-                ($file, $comment) = $line =~ /^(\S+)\s*(.*)/;
-            }
-
-            next unless $file;
-            
-            push @files, $file;
-        }
+        @files = _read_file( $skip_fh );
     }
 
     return \@files;
@@ -230,37 +267,30 @@ sub _read_skip {
 
 1;
 
+__END__
 
 =pod
 
 =head1 NAME
 
-Test::CheckManifest
+Test::CheckManifest - Check if your Manifest matches your distro
 
 =head1 VERSION
 
-version 1.23
+version 1.27
 
 =head1 SYNOPSIS
 
   use Test::CheckManifest;
   ok_manifest();
 
-=head1 DESCRIPTION
-
-C<Test::CheckManifest>
-
 =head2 EXPORT
 
 There is only one method exported: C<ok_manifest>
 
-=head1 NAME
-
-Test::CheckManifest - Check if your Manifest matches your distro
-
 =head1 METHODS
 
-=head2 ok_manifest   [{exlude => $arref}][$msg]
+=head2 ok_manifest   [{exclude => $arref}][$msg]
 
 checks whether the Manifest file matches the distro or not. To match a distro
 the Manifest has to name all files that come along with the distribution.
@@ -336,29 +366,14 @@ he reported some bugs to RT.
 
 =head1 AUTHOR
 
-Renee Baecker, E<lt>module@renee-baecker.deE<gt>
+Renee Baecker <reneeb@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006 - 2011 by Renee Baecker
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Artistic License 2.0
-
-=head1 AUTHOR
-
-Renee Baecker <module@renee-baecker.de>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2011 by Renee Baecker.
+This software is Copyright (c) 2014 by Renee Baecker.
 
 This is free software, licensed under:
 
-  The Artistic License 2.0
+  The Artistic License 2.0 (GPL Compatible)
 
 =cut
-
-
-__END__
-
